import org.garret.serme.*;

import java.util.*;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

public class TestSerialization implements ISerializable { 
    public TestSerialization next;
    public TestSerialization parent;
    public byte      i1;
    public short     i2;
    public char      c;
    public int       i4;
    public long      i8;
    public byte[]    ai1;
    public short[]   ai2;
    public char[]    ac;
    public int[]     ai4;
    public long[]    ai8;
    public Object[]  ao;
    public String[]  as;
    public Vector    v;
    public Hashtable h;
    public String    s;
    public Date      d;

    public void writeObject(IOutputStream out) { 
        out.writeObject(next);
        out.writeObject(parent);
        out.writeByte(i1);
        out.writeShort(i2);
        out.writeChar(c);
        out.writeInt(i4);
        out.writeLong(i8);
        out.writeArrayOfByte(ai1);
        out.writeArrayOfShort(ai2);
        out.writeArrayOfChar(ac);
        out.writeArrayOfInt(ai4);
        out.writeArrayOfLong(ai8);
        out.writeArrayOfObject(ao);
        out.writeArrayOfString(as);
        out.writeObject(v);
        out.writeObject(h);
        out.writeString(s);
        out.writeDate(d);
    }

    public void readObject(IInputStream in) {
        next = (TestSerialization)in.readObject();
        parent = (TestSerialization)in.readObject();
        i1 = in.readByte();
        i2 = in.readShort();
        c = in.readChar();
        i4 = in.readInt();
        i8 = in.readLong();
        ai1 = in.readArrayOfByte();
        ai2 = in.readArrayOfShort();
        ac = in.readArrayOfChar();
        ai4 = in.readArrayOfInt();
        ai8 = in.readArrayOfLong();
        ao = in.readArrayOfObject();
        as = in.readArrayOfString();
        v = (Vector)in.readObject();
        h = (Hashtable)in.readObject();
        s = in.readString();
        d = in.readDate();
    }

    static void check(boolean condition) { 
        if (!condition) { 
            throw new RuntimeException("Checkion failure");
        }
    }

    static TestSerialization createSet(int start, int len) { 
        TestSerialization owner = new TestSerialization();
        owner.v = new Vector();
        owner.h = new Hashtable();
        for (int i = 0; i < len; i++) { 
            int id = start + i;
            TestSerialization member = new TestSerialization();
            member.parent = owner;
            member.next = owner.next;
            owner.next = member;

            member.i1 = (byte)id;
            member.i2 = (short)id;
            member.c = (char)id;
            member.i4 = (int)id;
            member.i8 = (long)id;
            
            member.ai1 = new byte[2];
            member.ai1[0] = (byte)1;
            member.ai1[1] = (byte)2;

            member.ai2 = new short[1];
            member.ai2[0] = (short)1;

            member.ac = new char[2];
            member.ac[0] = (char)1;
            member.ac[1] = (char)2;

            member.ai4 = new int[1];
            member.ai4[0] = 1;

            member.ai8 = new long[1];
            member.ai8[0] = 1;

            member.ao = new Object[2];
            member.ao[0] = "elem";
            member.ao[1] = member;
            
            member.as = new String[2];
            member.as[0] = "elem1";
            member.as[1] = "elem2";

            member.s = "obj" + id;
            member.d = new Date(id);

            owner.v.addElement(member);
            owner.h.put(new Integer(id), member);
        }
        return owner;
    }

    static void checkSet(TestSerialization owner, int start, int len) { 
        TestSerialization member = owner.next;
        check(owner.s == null);
        check(owner.d == null);
        check(owner.parent == null);

        for (int i = len; --i >= 0; member = member.next) { 
            int id = start + i;
            check(member.parent == owner);

            check(member.i1 == (byte)id);
            check(member.i2 == (short)id);
            check(member.c == (char)id);
            check(member.i4 == (int)id);
            check(member.i8 == (long)id);

            check(member.ai1.length == 2);
            check(member.ai1[0] == (byte)1);
            check(member.ai1[1] == (byte)2);

            check(member.ai2.length == 1);
            check(member.ai2[0] == (short)1);

            check(member.ac.length == 2);
            check(member.ac[0] == (char)1);
            check(member.ac[1] == (char)2);

            check(member.ai4.length == 1);
            check(member.ai4[0] == 1);

            check(member.ai8.length == 1);
            check(member.ai8[0] == 1);

            check(member.ao.length == 2);
            check(member.ao[0].equals("elem"));
            check(member.ao[1] == member);
            
            check(member.as.length == 2);
            check(member.as[0].equals("elem1"));
            check(member.as[1].equals("elem2"));

            check(member.s.equals("obj" + id));
            check(member.d.getTime() == id);

            check(member.v == null);
            check(member.h == null);

            check(owner.v.elementAt(i) == member);
            check(owner.h.get(new Integer(id)) == member);
        }
        check(member == null);
    }

    static void fillStream(IOutputStream stream) {
        TestSerialization set = createSet(0, 10);
        stream.writeObject(set);
        stream.writeObject(set.next);
        stream.writeObject(createSet(10, 100));
        stream.writeInt(1);
        stream.writeString("End of stream");
        stream.close();
    }

    static void checkStream(IInputStream stream) { 
        TestSerialization owner = (TestSerialization)stream.readObject();
        checkSet(owner, 0, 10);
        TestSerialization member = (TestSerialization)stream.readObject();
        check(owner.next == member);
        owner = (TestSerialization)stream.readObject();
        checkSet(owner, 10, 100);
        check(stream.readInt() == 1);
        check(stream.readString().equals("End of stream")); 
        stream.close();
    }

    public static void main(String[] args) 
    { 
        ObjectOutputStream oos = new ObjectOutputStream();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BufferedObjectOutputStream boos = new BufferedObjectOutputStream(baos);
        
        fillStream(oos);
        fillStream(boos);
       
        ObjectInputStream ois = new ObjectInputStream(oos.toByteArray());
        ByteArrayInputStream bais1 = new ByteArrayInputStream(oos.toByteArray());
        ByteArrayInputStream bais2 = new ByteArrayInputStream(baos.toByteArray());
        BufferedObjectInputStream bois1 = new BufferedObjectInputStream(bais1);
        BufferedObjectInputStream bois2 = new BufferedObjectInputStream(bais2);

        checkStream(ois);
        checkStream(bois1);
        checkStream(bois2);
    }
}