package org.garret.serme;

import java.io.*;
import java.util.Vector;
import org.garret.serme.impl.*;

/**
 * Class used to fetch serialized objects. Unlike ObjectInputStream which reads data only
 * from byte array, this class is able to extract data from any input stream.
 * This class should be used by applications which need serialization mechanism, but do not 
 * require per-object manipulation (add/update/remove). This class
 * is able to extract objects, serialized by <code>ObjectDataOutputSream</code>
 */
public class BufferedObjectInputStream extends BufferedObjectDataInputStream {
    /**
     * Constructor of ObjectInputStream.
     * @param is input stream
     */
    public BufferedObjectInputStream(InputStream is) {
        super(is);
        objects = new Vector();
        classes = new Vector();
        objects.addElement(null);
    }

    protected Object getObject(int oid) { 
        return objects.elementAt(oid);
    }

    protected Object deserializeObject() { 
        String className = null;
        try { 
            int cid = in.readShort() & 0xFFFF;
            Class cls;
            if (cid == 0) { 
                className = Converter.unpackString(in);   
                cls = Class.forName(className);
                classes.addElement(cls);
            } else { 
                cls = (Class)classes.elementAt(cid-1);
            }
            Object obj = cls.newInstance();
            objects.addElement(obj);
            unpackObject(obj);
            return obj;
        } catch (ClassNotFoundException x) {
            throw new SermeException("Class " + className + " not found");
        } catch (IllegalAccessException x) { 
            throw new SermeException("Class of instantiated object is not accessible: " + x);
        } catch (InstantiationException x) { 
            throw new SermeException("Failed to instantiate object: " + x);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    protected Vector objects;
    protected Vector classes;
}