package org.garret.serme;

import java.io.*;
import java.util.Vector;
import org.garret.serme.impl.*;

/**
 * Class used to serialize objects. Unlike ObjectOutputStream which writes data only
 * to byte array, this class is able to write data to any output stream.
 * This class should be used by applications which need serialization mechanism, but do not 
 * require per-object manipulation (add/update/remove). Objects serialized by this class
 * can be extracted by both <code>ObjectDataInputSream</code> and  <code>BufferedObjectDataInputSream</code> classes
 */
public class BufferedObjectOutputStream extends BufferedObjectDataOutputStream {
    /**
     * Constructor of ObjectOutputStream.
     * @param os output stream
     */
    public BufferedObjectOutputStream(OutputStream os) {
        super(os);
        objToOid = new ObjToOid();
        classToOid = new ObjToOid();
    }

    protected void serializeObject(Object obj) { 
        try { 
            if (obj == null) { 
                out.writeByte(Types.OBJECT);
                out.writeInt(0);
            } else { 
                int oid = objToOid.get(obj);
                if (oid == 0) {
                    out.writeByte(Types.INSTANCE);
                    oid = objToOid.size() + 1;
                    objToOid.put(obj, oid);
                    Class c = obj.getClass();
                    int cid = classToOid.get(c);
                    if (cid == 0) { 
                        cid = classToOid.size()+1;            
                        classToOid.put(c, cid);
                        out.writeShort((short)0);
                        Converter.packString(out, c.getName());
                    } else { 
                        out.writeShort((short)cid);
                    }
                    packObject(obj);
                } else { 
                    out.writeByte(Types.OBJECT);
                    out.writeInt(oid);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public byte[] toByteArray() { 
        throw new SermeException("BufferedObjectOutputStream.toByteArray not implemented");
    }


    protected ObjToOid     objToOid;
    protected ObjToOid     classToOid;
}