package org.garret.serme;

import java.util.Date;

/**
 * Interface for fetching component of the objects.
 * Programmer should use methods of this interface while
 * implementation of </code>ISerializable.readObject(IInputStream)</code> method
 */
public interface IInputStream { 
    /**
     * Read boolean value
     * @return boolean field value 
     */
    boolean   readBoolean();    
    /**
     * Read byte value
     * @return byte field value 
     */
    byte      readByte();    
    /**
     * Read char value
     * @return char field value 
     */
    char      readChar();
    /**
     * Read short value
     * @return short field value 
     */
    short     readShort();    
    /**
     * Read int value
     * @return int field value 
     */
    int       readInt();    
    /**
     * Read long value
     * @return long field value 
     */
    long      readLong();    
    /**
     * Read string value
     * @return string field value 
     */
    String    readString();    
    /**
     * Read date value
     * @return <code>java.util.date</code> field value 
     */
    Date      readDate();    
    /**
     * Read object value. Supported object types include: 
     * <ul>
     * <li>all wrappers of builting Java types (<code>Integer</code>, <code>Long</code>,...)
     * <li>arrays of all builtin Jave types (<code>int[]</code>,...)
     * <li><code>java.util.Vector</code>
     * <li><code>java.util.Hashtable</code>
     * <li><code>java.util.Date</code>
     * <li><code>java.lang.String</code>
     * <li><code>java.util.Date[]</code>
     * <li><code>java.lang.String[]</code>
     * <li>any object implementing <code>ISerializable</code> interface
     * </ul>
     * @return <code>Object</code> field value 
     */
    Object    readObject();
    /**
     * Read array of boolean value
     * @return <code>boolean[]</code> field value 
     */
    boolean[] readArrayOfBoolean();    
    /**
     * Read array of char value
     * @return <code>char[]</code> field value 
     */
    byte[]    readArrayOfByte();    
    /**
     * Read array of byte value
     * @return <code>byte[]</code> field value 
     */
    char[]    readArrayOfChar();    
    /**
     * Read array of short value
     * @return <code>short[]</code> field value 
     */
    short[]   readArrayOfShort();    
    /**
     * Read array of int value
     * @return <code>int[]</code> field value 
     */
    int[]     readArrayOfInt();    
    /**
     * Read array of long value
     * @return <code>long[]</code> field value 
     */
    long[]    readArrayOfLong();    
    /**
     * Read array of String value
     * @return <code>String[]</code> field value 
     */
    String[]  readArrayOfString();    
    /**
     * Read array of Date value
     * @return <code>java.util.Date[]</code> field value 
     */
    Date[]    readArrayOfDate();    
    /**
     * Read array of object value. Restrictions of array elements type are the same as for 
     * <code>readObject()</code> method.
     * @return <code>Object[]</code> field value 
     */
    Object[]  readArrayOfObject();

    /**
     * Close the stream. 
     */
    void close();
}