package org.garret.serme;

import java.util.Date;

/**
 * Interface for storing component of the objects.
 * Programmer should use methods of this interface while
 * implementation of </code>ISerializable.writeObject(IOutputStream)</code> method
 */
public interface IOutputStream { 
    /**
     * Write boolean value
     * @param v value of the field
     */
    void writeBoolean(boolean v);    
    /**
     * Write byte value
     * @param v value of the field
     */
    void writeByte(byte v);    
    /**
     * Write char value
     * @param v value of the field
     */
    void writeChar(char v);
    /**
     * Write short value
     * @param v value of the field
     */
    void writeShort(short v);    
    /**
     * Write int value
     * @param v value of the field
     */
    void writeInt(int v);    
    /**
     * Write long value
     * @param v value of the field
     */
    void writeLong(long v);    
    /**
     * Write String value
     * @param v value of the field (may be null)
     */
    void writeString(String v);    
    /**
     * Write date value
     * @param v value of the field (may be null)
     */
    void writeDate(Date v);    
    /**
     * Write ob ject value. Supported object types include: 
     * <ul>
     * <li>all wrappers of builting Java types (<code>Integer</code>, <code>Long</code>,...)
     * <li>arrays of all builtin Jave types (<code>int[]</code>,...)
     * <li><code>java.util.Vector</code>
     * <li><code>java.util.Hashtable</code>
     * <li><code>java.util.Date</code>
     * <li><code>java.lang.String</code>
     * <li><code>java.util.Date[]</code>
     * <li><code>java.lang.String[]</code>
     * <li>any object implementing <code>ISerializable</code> interface
     * </ul>
     * @param v value of the field (may be null)
     */
    void writeObject(Object v);
    /**
     * Write array of boolean value
     * @param v value of the field (may be null)
     */
    void writeArrayOfBoolean(boolean[] v);    
    /**
     * Write array of byte value
     * @param v value of the field (may be null)
     */
    void writeArrayOfByte(byte[] v);    
    /**
     * Write array of char value
     * @param v value of the field (may be null)
     */
    void writeArrayOfChar(char[] v);    
    /**
     * Write array of short value
     * @param v value of the field (may be null)
     */
    void writeArrayOfShort(short[] v);    
    /**
     * Write array of ubt value
     * @param v value of the field (may be null)
     */
    void writeArrayOfInt(int[] v);    
    /**
     * Write array of long value
     * @param v value of the field (may be null)
     */
    void writeArrayOfLong(long[] v);    
    /**
     * Write array of string value
     * @param v value of the field (may be null)
     */
    void writeArrayOfString(String[] v);    
    /**
     * Write array of date value
     * @param v value of the field (may be null)
     */
    void writeArrayOfDate(Date[] v);    
    /**
     * Write array of object value. Restrictions of array elements type are the same as for 
     * <code>writeObject()</code> method.
     * @param v value of the field (may be null)
     */
    void writeArrayOfObject(Object[] v);  

    /**
     * Close the stream. It is not possible to write more data to the closed stream
     */
    void close();

    /**
     * Get byte array with serialized data. This method implicitly invoke close().
     * It is not possible to write more data to the stream after calling this method.
     * @return byte array with serialized objects
     */
    byte[] toByteArray();
}