package org.garret.serme;

import org.garret.serme.impl.*;
import java.util.Vector;

/**
 * Class used to fetch serialized objects.
 * This class should be used by applications which need serialization mechanism, but do not 
 * require per-object manipulation (add/update/remove). This class
 * is able to extract objects, serialized by <code>ObjectDataOutputSream</code>
 */
public class ObjectInputStream extends ObjectDataInputStream {
    /**
     * Constructor of ObjectInputStream.
     * @param arr byte array with serialized data
     */
    public ObjectInputStream(byte[] arr) {
        super(arr);
        objects = new Vector();
        classes = new Vector();
        objects.addElement(null);
    }

    protected Object getObject(int oid) { 
        return objects.elementAt(oid);
    }

    protected Object deserializeObject() { 
        String className = null;
        int cid = Converter.unpackShort(arr, pos) & 0xFFFF;
        pos += 2;
        try { 
            Class cls;
            if (cid == 0) { 
                className = Converter.unpackString(arr, pos);   
                pos += 2 + className.length()*2;
                cls = Class.forName(className);
                classes.addElement(cls);
            } else { 
                cls = (Class)classes.elementAt(cid-1);
            }
            Object obj = cls.newInstance();
            objects.addElement(obj);
            unpackObject(obj);
            return obj;
        } catch (ClassNotFoundException x) {
            throw new SermeException("Class " + className + " not found");
        } catch (IllegalAccessException x) { 
            throw new SermeException("Class of instantiated object is not accessible: " + x);
        } catch (InstantiationException x) { 
            throw new SermeException("Failed to instantiate object: " + x);
        }
    }

    protected Vector objects;
    protected Vector classes;
}