package org.garret.serme;

import java.util.*;
import org.garret.serme.impl.*;

/**
 * Class used to serialize objects into array of bytes.
 * This class should be used by applications which need serialization mechanism, but do not 
 * require per-object manipulation (add/update/remove). Object serialized by this class can be extracted 
 * by <code>ObjectDataInputSream</code>
 */
public class ObjectOutputStream extends ObjectDataOutputStream {
    /**
     * Constructor of ObjectOutputStream
     */
    public ObjectOutputStream() { 
        objToOid = new ObjToOid();
        classToOid = new ObjToOid();
    }
        
    public void close() {}

    protected void serializeObject(Object obj) { 
        if (obj == null) { 
            extend(5);
            arr[used++] = Types.OBJECT;
            Converter.packInt(arr, used, 0);
            used += 4;
        } else { 
            int oid = objToOid.get(obj);
            if (oid == 0) {
                extend(1);
                arr[used++] = Types.INSTANCE;
                oid = objToOid.size() + 1;
                objToOid.put(obj, oid);
                Class c = obj.getClass();
                int cid = classToOid.get(c);
                if (cid == 0) { 
                    cid = classToOid.size()+1;            
                    classToOid.put(c, cid);
                    String className = c.getName();
                    extend(4 + className.length()*2);
                    Converter.packShort(arr, used, (short)0);
                    used += 2;
                    Converter.packString(arr, used, className);
                    used += 2 + className.length()*2;
                } else { 
                    extend(2);
                    Converter.packShort(arr, used, (short)cid);
                    used += 2;
                }
                packObject(obj);
            } else { 
                extend(5);
                arr[used++] = Types.OBJECT;
                Converter.packInt(arr, used, oid);
                used += 4;
            }
        }
    }

    protected ObjToOid     objToOid;
    protected ObjToOid     classToOid;
}