package org.garret.serme;

/**
 * Class which remembers the storage to which it belongs. 
 * When objet is loaded from ther storage, the reference to this storage is saved in the object
 * using setStorage method. Using this reference to the storage, object can store or remove
 * himself from the storage.
 */
public abstract class Persistent implements ISerializable { 
    /**
     * Specify storage for the object. 
     * This method is mostly used by SerME internals and should not be directly 
     * invoked by programmer
     * @param storage associated storage
     */
    public void setStorage(IStorage storage) {
        this.storage = storage; 
    }

    /**
     * Get storage where object is stored 
     * @return storage containing object, <code>null</code> if object is not persistent
     */
    public IStorage getStorage() { 
        return storage;
    }
    
    /**
     * Save object in the storage (if any)
     */
    public void store() { 
        if (storage != null) { 
            storage.store(this);
        }
    }

    /**
     * Remove object from the storage
     */
    public void remove() { 
        if (storage != null) { 
            storage.remove(this);
            storage = null;
        }
    }

    private transient IStorage storage;
}
        


     