package org.garret.serme.file;

import org.garret.serme.*;
import org.garret.serme.impl.*;

import java.io.*;

class FileObjectInputStream extends ObjectDataInputStream {
    FileObjectInputStream() {
        super(null);
    }

    Object getRootObject(FileStorage storage) { 
        String className = null;
        try { 
            InputStream in = storage.con.openInputStream();
            int fileSize = (int)storage.con.fileSize();
            byte[] buf = new byte[fileSize];
            int size;
            int offs = 0;
            arr = buf;
            while (offs < fileSize) { 
                int rc = in.read(buf, offs, fileSize - offs);
                if (rc >= 0) { 
                    offs += rc;
                } else { 
                    throw new SermeException("Read file error");
                }
            }
            in.close();

            storage.rootOid = Converter.unpackInt(buf, FileStorage.DB_HDR_ROOT);
            OidToObj oidToObj = new OidToObj();
            this.oidToObj = oidToObj;
            int lastOid = 0;

            // First path: extract class descriptors
            for (offs = FileStorage.SIZE_OF_DB_HDR; offs < fileSize; offs += size) {
                size = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_SIZE);
                int oid = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_OID);
                if (oid != 0) { 
                    int cid = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_CID);
                    if (cid == 0) { 
                        // class descriptor
                        className = Converter.unpackString(buf, offs + FileStorage.SIZE_OF_OBJ_HDR);
                        oidToObj.put(oid, Class.forName(className));
                    }
                    if (oid > lastOid) { 
                        lastOid = oid;
                    }
                } else { 
                    storage.addHole(offs, size);
                }
            }
            storage.lastOid = lastOid;

            // Second path: instantiate objects
            for (offs = FileStorage.SIZE_OF_DB_HDR; offs < fileSize; offs += size) {
                size = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_SIZE);
                int oid = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_OID);
                if (oid != 0) { 
                    int cid = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_CID);
                    if (cid != 0) { 
                        Class c = (Class)oidToObj.get(cid);                        
                        Object obj = c.newInstance();
                        oidToObj.put(oid, obj);
                    }
                }
            }

            ObjToHdr objToHdr = new ObjToHdr(oidToObj.size());
            // Third path: unpack objects
            for (offs = FileStorage.SIZE_OF_DB_HDR; offs < fileSize; offs += size) {
                size = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_SIZE);
                int oid = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_OID);
                if (oid != 0) { 
                    Object obj = oidToObj.get(oid);
                    objToHdr.put(obj, oid, offs, size);
                    int cid = Converter.unpackInt(buf, offs + FileStorage.OBJ_HDR_CID);
                    if (cid != 0) { 
                        pos = offs + FileStorage.SIZE_OF_OBJ_HDR;
                        unpackObject(obj);
                        if (obj instanceof Persistent) {
                            ((Persistent)obj).setStorage(storage);
                        }
                    }
                }
            }
            storage.objToHdr = objToHdr;
            arr = null;
        } catch (IOException x) {
            throw new SermeException("File access error: " + x);
        } catch (ClassNotFoundException x) {
            throw new SermeException("Class " + className + " not found");
        } catch (IllegalAccessException x) { 
            throw new SermeException("Class of instantiated object is not accessible: " + x);
        } catch (InstantiationException x) { 
            throw new SermeException("Failed to instantiate object: " + x);
        }
        return oidToObj.get(storage.rootOid);
    }

    protected Object getObject(int oid) { 
        return oidToObj.get(oid);
    }

    protected Object deserializeObject() { 
        throw new SermeException("Invalid serialization data");
    }

    OidToObj oidToObj;
}
