package org.garret.serme.file;

import java.util.Vector;
import java.io.*;
import org.garret.serme.*;
import org.garret.serme.impl.*;


class FileObjectOutputStream extends ObjectDataOutputStream {

    public FileObjectOutputStream(FileStorage storage) { 
        this.storage = storage;
    }
        
    public void close() {}

    protected void serializeObject(Object obj) { 
        int oid;
        if (obj == null) { 
            oid = 0;
        } else { 
            ObjectHeader hdr = storage.objToHdr.get(obj);
            if (hdr == null) {
                oid = ++storage.lastOid;
                storage.objToHdr.put(obj, oid, 0, 0);
                objects.addElement(obj);
            } else { 
                oid = hdr.oid;
            }
        }
        extend(5);        
        arr[used++] = Types.OBJECT;
        Converter.packInt(arr, used, oid);
        used += 4;
    }

    void storeRoot(Object obj) { 
        storeObject(obj);
        ObjectHeader hdr = storage.objToHdr.get(obj);
        storage.rootOid = hdr.oid;
        byte[] metadata = new byte[FileStorage.SIZE_OF_DB_HDR];
        Converter.packInt(metadata, FileStorage.DB_HDR_ROOT, storage.rootOid);
        try { 
            OutputStream out = storage.con.openOutputStream();
            out.write(metadata);
            out.close();
        } catch (IOException x) { 
            throw new SermeException("Failed to store metadata: " + x);
        }
    }
            
    void storeObject(Object obj) { 
        objects = new Vector();
        storeObject0(obj);
        for (int i = 0; i < objects.size(); i++) { 
            storeObject0(objects.elementAt(i));
        }
        objects = null;
    }

    void storeObject0(Object obj) { 
        ObjectHeader hdr = storage.objToHdr.get(obj);
        int oid = 0;
        if (hdr == null) { 
            oid = ++storage.lastOid;
        } else { 
            oid = hdr.oid;
        }
        Class c = obj.getClass();
        int cid = storage.getClassId(c);
        extend(FileStorage.SIZE_OF_OBJ_HDR);
        Converter.packInt(arr, FileStorage.OBJ_HDR_OID, oid);
        Converter.packInt(arr, FileStorage.OBJ_HDR_CID, cid);
        used = FileStorage.SIZE_OF_OBJ_HDR;
        packObject(obj);
        int size = FileStorage.align(used);
        extend(size);
        Converter.packInt(arr, FileStorage.OBJ_HDR_SIZE, size);
        if (obj instanceof Persistent) { 
            ((Persistent)obj).setStorage(storage);
        }
        try { 
            int pos;
            if (hdr == null) { 
                pos = storage.allocate(size);
            } else { 
                if (size == hdr.size) { 
                    pos = hdr.offs;
                } else { 
                    if (hdr.size != 0) { 
                        storage.free(hdr.offs, hdr.size);
                    }
                    pos = storage.allocate(size);
                }
            }
            OutputStream out = storage.con.openOutputStream(pos);
            out.write(arr, 0, size);
            out.close();
            storage.objToHdr.put(obj, oid, pos, size);
        } catch (IOException x) { 
            throw new SermeException("Failed to store object: " + x);
        }
    }
    
    Vector      objects;
    FileStorage storage;
}