package org.garret.serme.file;
import java.util.*;

public class ObjToHdr { 
    ObjectHeader table[];
    static final int loadFactor = 75;
    int count;
    int threshold;

    public ObjToHdr() {
        this(1013);
    }

    public ObjToHdr(int threshold) {
        this.threshold = threshold;
        table = new ObjectHeader[threshold*100/loadFactor];
    }

    public void put(Object obj, int oid, int offs, int size) { 
        ObjectHeader tab[] = table;
        int h = System.identityHashCode(obj);
        int index = (h & 0x7FFFFFFF) % tab.length;
        for (ObjectHeader e = tab[index]; e != null; e = e.next) {
            if (e.obj == obj) {
                e.oid = oid;
                e.offs = offs;
                e.size = size;
                return;
            }
        }
        if (count >= threshold) {
            // Rehash the table if the threshold is exceeded
            rehash();
            tab = table;
            index = (h & 0x7FFFFFFF) % tab.length;
        } 

        // Creates the new entry.
        tab[index] = new ObjectHeader(obj, oid, offs, size, tab[index]);
        count++;
    }

    public ObjectHeader remove(Object obj) {
        ObjectHeader tab[] = table;
        int h = System.identityHashCode(obj);
        int index = (h & 0x7FFFFFFF) % tab.length;
        for (ObjectHeader e = tab[index], prev = null; e != null; prev = e, e = e.next) {
            if (e.obj == obj) {
                count -= 1;
                if (prev != null) {
                    prev.next = e.next;
                } else {
                    tab[index] = e.next;
                }
                return e;
            }
        }
        return null;
    }

    public ObjectHeader get(Object obj) {
        ObjectHeader tab[] = table;
        int h = System.identityHashCode(obj);
        int index = (h & 0x7FFFFFFF) % tab.length;
        for (ObjectHeader e = tab[index]; e != null; e = e.next) {
            if (e.obj == obj) {
                return e;
            }
        }
        return null;
    }
    
    int size() { 
        return count;
    }

    void rehash() {
        int oldCapacity = table.length;
        ObjectHeader oldMap[] = table;
        int i;

        int newCapacity = oldCapacity * 2 + 1;
        ObjectHeader newMap[] = new ObjectHeader[newCapacity];

        threshold = (int)(newCapacity * loadFactor);
        table = newMap;

        for (i = oldCapacity; --i >= 0;) {
            for (ObjectHeader old = oldMap[i]; old != null; ) {
                ObjectHeader e = old;
                old = old.next;

                int index = (System.identityHashCode(e.obj) & 0x7FFFFFFF) % newCapacity;
                e.next = newMap[index];
                newMap[index] = e;
            }
        }
    }
    
    class ObjectEnumeration implements Enumeration { 
        ObjectEnumeration(ObjectHeader[] table) {             
            for (int i = 0; i < table.length; i++) { 
                for (ObjectHeader hdr = table[i]; hdr != null; hdr = hdr.next) {
                    if (!(hdr.obj instanceof Class)) {
                        curr = hdr;
                        currIndex = i;
                        return;
                    }
                }
            }
        }
    
        public boolean hasMoreElements() { 
            return curr != null;
        }
        
        public Object nextElement() { 
            if (curr == null) { 
                throw new NoSuchElementException();
            }
            Object obj = curr.obj; 
            while ((curr = curr.next) != null) { 
                if (!(curr.obj instanceof Class)) { 
                    return obj;
                }
            }
            for (int i = currIndex; ++i < table.length;) { 
                for (ObjectHeader hdr = table[i]; hdr != null; hdr = hdr.next) {
                    if (!(hdr.obj instanceof Class)) {
                        curr = hdr;
                        currIndex = i;
                        return obj;
                    }
                }
            }
            return obj;
        }
        
        int            currIndex;
        ObjectHeader[] table;
        ObjectHeader   curr;
    }

    public Enumeration keys() { 
        return new ObjectEnumeration(table);
    }
}







