package org.garret.serme.impl;

import java.io.*;
import java.util.*;
import org.garret.serme.*;

public abstract class BufferedObjectDataInputStream implements IInputStream {
    protected abstract Object getObject(int oid);
    protected abstract Object deserializeObject();

    protected void check(int type) {         
        try { 
            int t = in.readByte();
            if (t != type) { 
                throw new SermeException("Expected field type " + Types.MNEM[type] + ", actual type " + Types.MNEM[t]);
            } 
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public boolean readBoolean() { 
        try { 
            check(Types.BOOLEAN);
            return in.readBoolean();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public byte readByte() { 
        try { 
            check(Types.BYTE);
            return in.readByte();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public char readChar() {
        try { 
            check(Types.CHAR);
            return in.readChar();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public short readShort() { 
        try { 
            check(Types.SHORT);
            return in.readShort();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public int readInt() { 
        try { 
            check(Types.INT);
            return in.readInt();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public long readLong() { 
        try { 
            check(Types.LONG);
            return in.readLong();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public String readString() { 
        try { 
            check(Types.STRING);
            return Converter.unpackString(in);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public Date readDate() { 
        try { 
            check(Types.DATE);
            return Converter.unpackDate(in);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

        
    public  boolean[] readArrayOfBoolean() { 
        try { 
            check(Types.ARRAY_OF_BOOLEAN);
            return readArrayOfBoolean0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final boolean[] readArrayOfBoolean0() throws IOException { 
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        boolean[] v = new boolean[size];
        for (int i = 0; i < size; i++) { 
            v[i] = in.readBoolean();
        }
        return v;
    }
        
    public byte[] readArrayOfByte() {
        try { 
            check(Types.ARRAY_OF_BYTE);
            return readArrayOfByte0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final byte[] readArrayOfByte0() throws IOException {
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        byte[] v = new byte[size];
        for (int i = 0; i < size; i++) { 
            v[i] = in.readByte();
        }
        return v;
    }
        

    public char[] readArrayOfChar() {
        try { 
            check(Types.ARRAY_OF_CHAR);
            return readArrayOfChar0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final char[] readArrayOfChar0() throws IOException {
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        char[] v = new char[size];
        for (int i = 0; i < size; i++) { 
            v[i] = in.readChar();
        }
        return v;
    }
       
    public short[] readArrayOfShort() { 
        try { 
            check(Types.ARRAY_OF_SHORT);
            return readArrayOfShort0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final short[] readArrayOfShort0() throws IOException { 
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        short[] v = new short[size];
        for (int i = 0; i < size; i++) { 
            v[i] = in.readShort();
        }
        return v;
    }
        
    public int[] readArrayOfInt() {
        try { 
            check(Types.ARRAY_OF_INT);
            return readArrayOfInt0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final int[] readArrayOfInt0() throws IOException {
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        int[] v = new int[size];
        for (int i = 0; i < size; i++) { 
            v[i] = in.readInt();
        }
        return v;
    }
        
    public long[] readArrayOfLong() { 
        try { 
            check(Types.ARRAY_OF_LONG);
            return readArrayOfLong0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final long[] readArrayOfLong0() throws IOException { 
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        long[] v = new long[size];
        for (int i = 0; i < size; i++) { 
            v[i] = in.readLong();
        }
        return v;
    }
 
    public String[] readArrayOfString() { 
        try { 
            check(Types.ARRAY_OF_STRING);
            return readArrayOfString0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final String[] readArrayOfString0() throws IOException { 
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        String[] v = new String[size];
        for (int i = 0; i < size; i++) { 
            v[i] = Converter.unpackString(in);            
        }
        return v;
    }
 
    public Date[] readArrayOfDate() { 
        try { 
            check(Types.ARRAY_OF_DATE);
            return readArrayOfDate0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }
    
    private final Date[] readArrayOfDate0() throws IOException { 
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        Date[] v = new Date[size];
        for (int i = 0; i < size; i++) { 
            v[i] = Converter.unpackDate(in);
        }
        return v;
    }
 
    public Object[] readArrayOfObject() { 
        try { 
            check(Types.ARRAY_OF_OBJECT);
            return readArrayOfObject0();
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    private final Object[] readArrayOfObject0() throws IOException { 
        int size = in.readInt();
        if (size < 0) { 
            return null;
        }
        Object[] v = new Object[size];
        for (int i = 0; i < size; i++) { 
            v[i] = readObject();
        }
        return v;
    }

    protected void unpackObject(Object obj) throws IOException {
        if (obj instanceof ISerializable) { 
            ((ISerializable)obj).readObject(this);
        } else if (obj instanceof Vector) { 
            Vector v = (Vector)obj;
            int size = in.readInt();
            while (--size >= 0) { 
                v.addElement(readObject());
            }
        } else if (obj instanceof Hashtable) { 
            Hashtable h = (Hashtable)obj;
            int size = in.readInt();
            while (--size >= 0) { 
                Object key = readObject();
                Object value = readObject();
                h.put(key, value);
            }
        } else { 
            throw new SermeException("Bad object type");
        }
    }

    public Object readObject() { 
        try { 
            int type = in.readByte();
            switch (type) { 
            case Types.BOOLEAN:
                return new Boolean(in.readBoolean());
            case Types.BYTE:
                return new Byte(in.readByte());
            case Types.CHAR:
                return new Character(in.readChar());
            case Types.SHORT:
                return new Short(in.readShort());
            case Types.INT:
                return new Integer(in.readInt());
            case Types.LONG:
                return new Long(in.readLong());
            case Types.STRING :
                return Converter.unpackString(in);
            case Types.DATE:
                return Converter.unpackDate(in);
            case Types.INSTANCE:
                return deserializeObject();
            case Types.OBJECT:
                return getObject(in.readInt());
            case Types.ARRAY_OF_BOOLEAN:
                return readArrayOfBoolean0();
            case Types.ARRAY_OF_BYTE:
                return readArrayOfByte0();
            case Types.ARRAY_OF_SHORT:
                return readArrayOfShort0();
            case Types.ARRAY_OF_CHAR:
                return readArrayOfChar0();
            case Types.ARRAY_OF_INT:
                return readArrayOfInt0();
            case Types.ARRAY_OF_LONG:
                return readArrayOfLong0();
            case Types.ARRAY_OF_STRING:
                return readArrayOfString0();
            case Types.ARRAY_OF_DATE:
                return readArrayOfDate0();            
            case Types.ARRAY_OF_OBJECT:
                return readArrayOfObject0();            
            default:
                throw new SermeException("Unexpected field type: " + Types.MNEM[type]);
            }            
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }
    
    public void close() { 
        try { 
            in.close();
        } catch (IOException x) {
            throw new SermeException("Close error: " + x);
        }
    }

    protected BufferedObjectDataInputStream(InputStream is) { 
        in = new DataInputStream(is);
    }
 
    protected DataInputStream in;
}