package org.garret.serme.impl;

import java.io.*;
import java.util.*;
import org.garret.serme.*;

public abstract class BufferedObjectDataOutputStream implements IOutputStream {
    protected abstract void serializeObject(Object obj);

    public void writeByte(byte v) {        
        try { 
            out.writeByte(Types.BYTE);
            out.writeByte(v);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public void writeBoolean(boolean v) { 
        try { 
            out.writeByte(Types.BOOLEAN);
            out.writeByte((byte)(v ? 1 : 0));
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public void writeChar(char v) { 
        try { 
            out.writeByte(Types.CHAR);
            out.writeChar(v);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public void writeShort(short v) { 
        try { 
            out.writeByte(Types.SHORT);
            out.writeShort(v);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public void writeInt(int v) { 
        try { 
            out.writeByte(Types.INT);
            out.writeInt(v);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public void writeLong(long v) { 
        try { 
            out.writeByte(Types.LONG);
            out.writeLong(v);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public void writeString(String s) { 
        try { 
            out.writeByte(Types.STRING);
            Converter.packString(out, s);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }

    public void writeDate(Date v) { 
        try { 
            out.writeByte(Types.DATE);
            Converter.packDate(out, v);
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }


    public void writeArrayOfBoolean(boolean[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_BOOLEAN);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                byte[] bits = new byte[(len + 7) >> 3];
                for (int i = 0; i < len; i++) { 
                    bits[i >>> 3] |= 1 << (i & 7);
                }
                out.write(bits);
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    

    public void writeArrayOfByte(byte[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_BYTE);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                out.writeInt(v.length);
                out.write(v);
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    
    public void writeArrayOfShort(short[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_SHORT);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                for (int i = 0; i < len; i++) { 
                    out.writeShort(v[i]);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    
    public void writeArrayOfChar(char[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_CHAR);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                for (int i = 0; i < len; i++) { 
                    out.writeChar(v[i]);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    
    
    public void writeArrayOfInt(int[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_INT);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                for (int i = 0; i < len; i++) { 
                    out.writeInt(v[i]);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    

    public void writeArrayOfLong(long[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_LONG);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                for (int i = 0; i < len; i++) { 
                    out.writeLong(v[i]);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    
    public void writeArrayOfString(String[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_STRING);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                for (int i = 0; i < len; i++) { 
                    Converter.packString(out, v[i]);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    
    public void writeArrayOfDate(Date[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_DATE);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                for (int i = 0; i < len; i++) { 
                    Converter.packDate(out, v[i]);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        

    public void writeArrayOfObject(Object[] v) { 
        try { 
            out.writeByte(Types.ARRAY_OF_OBJECT);
            if (v == null) { 
                out.writeInt(-1);
            } else { 
                int len = v.length;
                out.writeInt(len);
                for (int i = 0; i < len; i++) { 
                    writeObject(v[i]);
                }
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        
    
    protected void packObject(Object obj) { 
        try { 
            if (obj instanceof ISerializable) { 
                ((ISerializable)obj).writeObject(this);
            } else if (obj instanceof Vector) { 
                Vector v = (Vector)obj;
                int n = v.size();
                out.writeInt(n);
                for (int i = 0; i < n; i++) { 
                    writeObject(v.elementAt(i));
                }
            } else if (obj instanceof Hashtable) { 
                Hashtable h = (Hashtable)obj;
                int n = h.size();
                out.writeInt(n);
                Enumeration e = h.keys();
                while (e.hasMoreElements()) { 
                    Object key = e.nextElement();
                    writeObject(key);
                    writeObject(h.get(key));
                }
            } else { 
                throw new SermeException("Bad object type");
            }
        } catch (IOException x) { 
            throw new SermeException(x);
        }
    }        

    public void writeObject(Object obj) {
        if (obj == null || obj instanceof ISerializable || obj instanceof Vector || obj instanceof Hashtable) { 
            serializeObject(obj);
        } else if (obj instanceof Boolean) { 
            writeBoolean(((Boolean)obj).booleanValue());
        } else if (obj instanceof Byte) { 
            writeByte(((Byte)obj).byteValue());
        } else if (obj instanceof Short) { 
            writeShort(((Short)obj).shortValue());
        } else if (obj instanceof Integer) { 
            writeInt(((Integer)obj).intValue());
        } else if (obj instanceof Long) { 
            writeLong(((Long)obj).longValue());
        } else if (obj instanceof String) { 
            writeString((String)obj);
        } else if (obj instanceof Date) { 
            writeDate((Date)obj);
        } else if (obj instanceof boolean[]) { 
            writeArrayOfBoolean((boolean[])obj);
        } else if (obj instanceof byte[]) { 
            writeArrayOfByte((byte[])obj);
        } else if (obj instanceof short[]) { 
            writeArrayOfShort((short[])obj);
        } else if (obj instanceof char[]) { 
            writeArrayOfChar((char[])obj);
        } else if (obj instanceof int[]) { 
            writeArrayOfInt((int[])obj);
        } else if (obj instanceof long[]) { 
            writeArrayOfLong((long[])obj);
        } else if (obj instanceof String[]) { 
            writeArrayOfString((String[])obj);
        } else if (obj instanceof Date[]) { 
            writeArrayOfDate((Date[])obj);
         } else if (obj instanceof Object[]) { 
            writeArrayOfObject((Object[])obj);
       } else { 
            throw new SermeException("Object of type " + obj.getClass() + " can not be serialized");
        }
    }

    public void close() { 
        try { 
            out.close();
        } catch (IOException x) {
            throw new SermeException("Close error: " + x);
        }
    }

    protected BufferedObjectDataOutputStream(OutputStream os) { 
        out = new DataOutputStream(os);
    }

    protected DataOutputStream out;
}