package org.garret.serme.impl;

import java.util.Date;
import java.io.*;

public class Converter {
    public static void packShort(byte[] arr, int offs, short x) { 
        arr[offs] = (byte)(x >> 8);
        arr[offs+1] = (byte)x;
    }

    public static short unpackShort(byte[] arr, int offs) { 
        return (short)(((arr[offs] & 0xFF) << 8) | (arr[offs+1] & 0xFF));
    }

    public static void packChar(byte[] arr, int offs, char x) { 
        arr[offs] = (byte)(x >> 8);
        arr[offs+1] = (byte)x;
    }
  
    public static char unpackChar(byte[] arr, int offs) { 
        return (char)(((arr[offs] & 0xFF) << 8) | (arr[offs+1] & 0xFF));
    }

    public static void packBoolean(byte[] arr, int offs, boolean x) { 
        arr[offs] = (byte)(x ? 1 : 0);
    }

    public static boolean unpackBoolean(byte[] arr, int offs) { 
        return arr[offs] != 0;
    }

    public static void packInt(byte[] arr, int offs, int x) { 
        arr[offs] = (byte)(x >> 24);
        arr[offs+1] = (byte)(x >> 16);
        arr[offs+2] = (byte)(x >> 8);
        arr[offs+3] = (byte)x;
    }

    public static int unpackInt(byte[] arr, int offs) { 
        return (arr[offs] << 24) | ((arr[offs+1] & 0xFF) << 16) | ((arr[offs+2] & 0xFF) << 8) | (arr[offs+3] & 0xFF);
    }

    public static void packLong(byte[] arr, int offs, long x) { 
        packInt(arr, offs, (int)(x >> 32));
        packInt(arr, offs+4, (int)x);
    }

    public static long unpackLong(byte[] arr, int offs) { 
        return ((long)unpackInt(arr, offs) << 32) | (unpackInt(arr, offs+4) & 0xFFFFFFFFL);
    }

    public static void packDate(byte[] arr, int offs, Date x) { 
        packLong(arr, offs, x == null ? -1L : x.getTime());
    }

    public static void packDate(DataOutputStream out, Date x) throws IOException { 
        out.writeLong(x == null ? -1L : x.getTime());
    }            

    public static Date unpackDate(byte[] arr, int offs) { 
        long t = unpackLong(arr, offs);
        return t < 0 ? (Date)null : new Date(t);
    }

    public static Date unpackDate(DataInputStream in) throws IOException { 
        long t = in.readLong();
        return t < 0 ? (Date)null : new Date(t);
    }

    public static void packString(byte[] arr, int offs, String str) { 
        if (str == null) { 
            packShort(arr, offs, (short)-1);
        } else { 
            int n = str.length();
            packShort(arr, offs, (short)n);
            offs += 2; 
            for (int i = 0; i < n; i++) { 
                packChar(arr, offs, str.charAt(i));
                offs += 2;
            }
        }
    }
            
    public static void packString(DataOutputStream out, String str) throws IOException { 
        if (str == null) { 
            out.writeShort((short)-1);
        } else { 
            int len = str.length();
            out.writeShort((short)len);
            for (int i = 0; i < len; i++) { 
                out.writeChar(str.charAt(i));
            }
        }
    }

    public static String unpackString(byte[] arr, int offs) { 
        int n = unpackShort(arr, offs);        
        offs += 2;         
        if (n < 0) { 
            return null;
        }
        char[] chars = new char[n];
        for (int i = 0; i < n; i++) { 
            chars[i] = unpackChar(arr, offs);
            offs += 2;
        }
        return new String(chars);
    }

    public static String unpackString(DataInputStream in) throws IOException { 
        int n = in.readShort();        
        if (n < 0) { 
            return null;
        }
        char[] chars = new char[n];
        for (int i = 0; i < n; i++) { 
            chars[i] = in.readChar();
        }
        return new String(chars);
    }
}
