package org.garret.serme.impl;

import java.util.*;

public class ObjToOid { 
    HashEntry table[];
    static final int loadFactor = 75;
    int count;
    int threshold;

    public ObjToOid() {
        this(11);
    }

    public ObjToOid(int threshold) {
        this.threshold = threshold;
        table = new HashEntry[threshold*100/loadFactor];
    }

    public ObjToOid(OidToObj hash) {
        this(hash.size());
        for (int i = 0; i < hash.table.length; i++) { 
            for (HashEntry e = hash.table[i]; e != null; e = e.next) {
                put(e.obj, e.oid);
            }
        }
    }

    public int size() { 
        return count;
    }

    public int remove(Object obj) {
        HashEntry tab[] = table;
        int h = System.identityHashCode(obj);
        int index = (h & 0x7FFFFFFF) % tab.length;
        for (HashEntry e = tab[index], prev = null; e != null; prev = e, e = e.next) {
            if (e.obj == obj) {
                count -= 1;
                if (prev != null) {
                    prev.next = e.next;
                } else {
                    tab[index] = e.next;
                }
                return e.oid;
            }
        }
        return 0;
    }
 
    public void put(Object obj, int oid) { 
        HashEntry tab[] = table;
        int h = System.identityHashCode(obj);
        int index = (h & 0x7FFFFFFF) % tab.length;
        for (HashEntry e = tab[index]; e != null; e = e.next) {
            if (e.obj == obj) {
                e.oid = oid;
                return;
            }
        }
        if (count >= threshold) {
            // Rehash the table if the threshold is exceeded
            rehash();
            tab = table;
            index = (h & 0x7FFFFFFF) % tab.length;
        } 

        // Creates the new entry.
        tab[index] = new HashEntry(oid, obj, tab[index]);
        count++;
    }

    public int get(Object obj) {
        HashEntry tab[] = table;
        int h = System.identityHashCode(obj);
        int index = (h & 0x7FFFFFFF) % tab.length;
        for (HashEntry e = tab[index]; e != null ; e = e.next) {
            if (e.obj == obj) {
                return e.oid;
            }
        }
        return 0;
    }
    
    void rehash() {
        int oldCapacity = table.length;
        HashEntry oldMap[] = table;
        int i;

        int newCapacity = oldCapacity * 2 + 1;
        HashEntry newMap[] = new HashEntry[newCapacity];

        threshold = (int)(newCapacity * loadFactor);
        table = newMap;

        for (i = oldCapacity; --i >= 0 ;) {
            for (HashEntry old = oldMap[i]; old != null; ) {
                HashEntry e = old;
                old = old.next;

                int index = (System.identityHashCode(e.obj) & 0x7FFFFFFF) % newCapacity;
                e.next = newMap[index];
                newMap[index] = e;
            }
        }
    }

    public Enumeration keys() { 
        return new KeyEnumeration(table);
    }
}







