package org.garret.serme.impl;

import java.util.*;
import org.garret.serme.*;

public abstract class ObjectDataOutputStream extends ByteBuffer implements IOutputStream {
    protected abstract void serializeObject(Object obj);

    public void writeByte(byte v) { 
        extend(2);
        arr[used++] = Types.BYTE;
        arr[used++] = v;
    }

    public void writeBoolean(boolean v) { 
        extend(2);
        arr[used++] = Types.BOOLEAN;
        Converter.packBoolean(arr, used, v);
        used += 1;
    }

    public void writeChar(char v) { 
        extend(3);
        arr[used++] = Types.CHAR;
        Converter.packChar(arr, used, v);
        used += 2;
    }

    public void writeShort(short v) { 
        extend(3);
        arr[used++] = Types.SHORT;
        Converter.packShort(arr, used, v);
        used += 2;
    }

    public void writeInt(int v) { 
        extend(5);
        arr[used++] = Types.INT;
        Converter.packInt(arr, used, v);
        used += 4;
    }

    public void writeLong(long v) { 
        extend(9);
        arr[used++] = Types.LONG;
        Converter.packLong(arr, used, v);
        used += 8;
    }

    public void writeString(String s) { 
        if (s == null) { 
            extend(3);
            arr[used++] = Types.STRING;
            Converter.packShort(arr, used, (short)-1);
            used += 2;
        } else { 
            int len = s.length();
            extend(1 + (1+len)*2);            
            arr[used++] = Types.STRING;
            Converter.packString(arr, used, s);
            used += (1+len)*2;
        }
    }

    public void writeDate(Date v) { 
        extend(9);
        arr[used++] = Types.DATE;
        Converter.packDate(arr, used, v);
        used += 8;
    }


    public void writeArrayOfBoolean(boolean[] v) { 
        if (v == null) { 
            extend(5);
            arr[used++] = Types.ARRAY_OF_BOOLEAN;
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            extend(5 + v.length);
            arr[used++] = Types.ARRAY_OF_BOOLEAN;
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) { 
                Converter.packBoolean(arr, used++, v[i]);
            }
        }
    }        
    

    public void writeArrayOfByte(byte[] v) { 
        if (v == null) { 
            extend(5);
            arr[used++] = Types.ARRAY_OF_BYTE;
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            extend(5 + v.length);
            arr[used++] = Types.ARRAY_OF_BYTE;
            Converter.packInt(arr, used, v.length);
            used += 4;
            System.arraycopy(v, 0, arr, used, v.length);
            used += v.length;
        }
    }        
    
    public void writeArrayOfShort(short[] v) { 
        if (v == null) { 
            extend(5);
            arr[used++] = Types.ARRAY_OF_SHORT;
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            extend(5 + v.length*2);
            arr[used++] = Types.ARRAY_OF_SHORT;
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) { 
                Converter.packShort(arr, used, v[i]);
                used += 2;
            }
        }
    }        
    
    public void writeArrayOfChar(char[] v) { 
        if (v == null) { 
            extend(5);
            arr[used++] = Types.ARRAY_OF_CHAR;
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            extend(5 + v.length*2);
            arr[used++] = Types.ARRAY_OF_CHAR;
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) { 
                Converter.packChar(arr, used, v[i]);
                used += 2;
            }
        }
    }        
    
    
    public void writeArrayOfInt(int[] v) { 
        if (v == null) { 
            extend(5);
            arr[used++] = Types.ARRAY_OF_INT;
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            extend(5 + v.length*4);
            arr[used++] = Types.ARRAY_OF_INT;
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) { 
                Converter.packInt(arr, used, v[i]);
                used += 4;
            }
        }
    }        
    

    public void writeArrayOfLong(long[] v) { 
        if (v == null) { 
            extend(5);
            arr[used++] = Types.ARRAY_OF_LONG;
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            extend(5 + v.length);
            arr[used++] = Types.ARRAY_OF_LONG;
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) { 
                Converter.packLong(arr, used, v[i]);
                used += 8;
            }
        }
    }        
    
    public void writeArrayOfString(String[] v) { 
        extend(5);
        arr[used++] = Types.ARRAY_OF_STRING;
        if (v == null) { 
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) {
                String s = v[i];
                if (s == null) { 
                    extend(2);
                    Converter.packShort(arr, used, (short)-1);
                    used += 2;
                } else { 
                    int len = s.length();
                    extend((1+len)*2);            
                    Converter.packString(arr, used, s);
                    used += (1+len)*2;
                }
            }
        }
    }        
    
    public void writeArrayOfDate(Date[] v) { 
        if (v == null) { 
            extend(5);
            arr[used++] = Types.ARRAY_OF_DATE;
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            extend(5 + v.length*8);
            arr[used++] = Types.ARRAY_OF_DATE;
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) { 
                Converter.packDate(arr, used, v[i]);
                used += 8;
            }
        }
    }        

    public void writeArrayOfObject(Object[] v) { 
        extend(5);
        arr[used++] = Types.ARRAY_OF_OBJECT;
        if (v == null) { 
            Converter.packInt(arr, used, -1);
            used += 4;
        } else { 
            Converter.packInt(arr, used, v.length);
            used += 4;
            for (int i = 0; i < v.length; i++) {
                writeObject(v[i]);
            }
        }
    }        
    
    protected void packObject(Object obj) { 
        if (obj instanceof ISerializable) { 
            ((ISerializable)obj).writeObject(this);
        } else if (obj instanceof Vector) { 
            extend(4);
            Vector v = (Vector)obj;
            Converter.packInt(arr, used, v.size());
            used += 4;
            for (int i = 0, n = v.size(); i < n; i++) { 
                writeObject(v.elementAt(i));
            }
        } else if (obj instanceof Hashtable) { 
            extend(4);
            Hashtable h = (Hashtable)obj;
            Converter.packInt(arr, used, h.size());
            used += 4;
            Enumeration e = h.keys();
            while (e.hasMoreElements()) { 
                Object key = e.nextElement();
                writeObject(key);
                writeObject(h.get(key));
            }
        } else { 
            throw new SermeException("Bad object type");
        }
    }        

    public void writeObject(Object obj) {
        if (obj == null || obj instanceof ISerializable || obj instanceof Vector || obj instanceof Hashtable) { 
            serializeObject(obj);
        } else if (obj instanceof Boolean) { 
            writeBoolean(((Boolean)obj).booleanValue());
        } else if (obj instanceof Byte) { 
            writeByte(((Byte)obj).byteValue());
        } else if (obj instanceof Short) { 
            writeShort(((Short)obj).shortValue());
        } else if (obj instanceof Integer) { 
            writeInt(((Integer)obj).intValue());
        } else if (obj instanceof Long) { 
            writeLong(((Long)obj).longValue());
        } else if (obj instanceof String) { 
            writeString((String)obj);
        } else if (obj instanceof Date) { 
            writeDate((Date)obj);
        } else if (obj instanceof boolean[]) { 
            writeArrayOfBoolean((boolean[])obj);
        } else if (obj instanceof byte[]) { 
            writeArrayOfByte((byte[])obj);
        } else if (obj instanceof short[]) { 
            writeArrayOfShort((short[])obj);
        } else if (obj instanceof char[]) { 
            writeArrayOfChar((char[])obj);
        } else if (obj instanceof int[]) { 
            writeArrayOfInt((int[])obj);
        } else if (obj instanceof long[]) { 
            writeArrayOfLong((long[])obj);
        } else if (obj instanceof String[]) { 
            writeArrayOfString((String[])obj);
        } else if (obj instanceof Date[]) { 
            writeArrayOfDate((Date[])obj);
         } else if (obj instanceof Object[]) { 
            writeArrayOfObject((Object[])obj);
       } else { 
            throw new SermeException("Object of type " + obj.getClass() + " can not be serialized");
        }
    }
}