package org.garret.serme.impl;

public class OidToObj { 
    HashEntry table[];
    static final int loadFactor = 75;
    int count;
    int threshold;

    public OidToObj() {
        this(11);
    }

    public OidToObj(ObjToOid hash) {
        this(hash.size());
        for (int i = 0; i < hash.table.length; i++) { 
            for (HashEntry e = hash.table[i]; e != null; e = e.next) {
                put(e.oid, e.obj);
            }
        }
    }

    public OidToObj(int threshold) {
        this.threshold = threshold;
        table = new HashEntry[threshold*100/loadFactor];
    }

    public void put(int oid, Object obj) { 
        HashEntry tab[] = table;
        int index = (oid & 0x7FFFFFFF) % tab.length;
        for (HashEntry e = tab[index]; e != null; e = e.next) {
            if (e.oid == oid) {
                e.obj = obj;
                return;
            }
        }
        if (count >= threshold) {
            // Rehash the table if the threshold is exceeded
            rehash();
            tab = table;
            index = (oid & 0x7FFFFFFF) % tab.length;
        } 

        // Creates the new entry.
        tab[index] = new HashEntry(oid, obj, tab[index]);
        count++;
    }

    public Object get(int oid) {
        HashEntry tab[] = table;
        int index = (oid & 0x7FFFFFFF) % tab.length;
        for (HashEntry e = tab[index] ; e != null ; e = e.next) {
            if (e.oid == oid) {
                return e.obj;
            }
        }
        return null;
    }
    
    public int size() { 
        return count;
    }

    void rehash() {
        int oldCapacity = table.length;
        HashEntry oldMap[] = table;
        int i;

        int newCapacity = oldCapacity * 2 + 1;
        HashEntry newMap[] = new HashEntry[newCapacity];

        threshold = (int)(newCapacity * loadFactor);
        table = newMap;

        for (i = oldCapacity; --i >= 0 ;) {
            for (HashEntry old = oldMap[i]; old != null; ) {
                HashEntry e = old;
                old = old.next;

                int index = (e.oid & 0x7FFFFFFF) % newCapacity;
                e.next = newMap[index];
                newMap[index] = e;
            }
        }
    }
}







