package org.garret.serme.impl;

import java.util.*;
import org.garret.serme.*;

public class TtreeEnumeration implements Enumeration { 
    private int counter;
    private SortedCollection tree;
    private TtreePage[] pageStack;
    private int[]       posStack;
    private int         sp;

    private static final int MAX_HEIGHT = 8;

    public TtreeEnumeration(SortedCollection tree, TtreePage root) { 
        this.tree = tree;
        counter = tree.getCounter();
        pageStack = new TtreePage[MAX_HEIGHT];
        posStack = new int[MAX_HEIGHT];
        int i = 0;
        for (TtreePage page = root; page != null; page = page.left) { 
            pageStack[i++] = page;
        }
        sp = i;
    }

    public boolean hasMoreElements() { 
        if (counter != tree.getCounter()) { 
            throw new SermeException("Concurrent modification");
        }
        return sp != 0;
    }

    public Object nextElement() {
        if (!hasMoreElements()) { 
            throw new NoSuchElementException();
        }
        Object obj = pageStack[sp-1].item[posStack[sp-1]];
        TtreePage page = pageStack[sp-1];
        if (++posStack[sp-1] == page.nItems) { 
            if (page.right != null) { 
                page = page.right;
                int i = sp;
                do { 
                    pageStack[i] = page;
                    posStack[i] = 0;
                    i += 1;
                } while ((page = page.left) != null);
                sp = i;
            } else {                     
                while (--sp > 0 && posStack[sp-1] == pageStack[sp-1].nItems);
            }
        }
        return obj;
    }
}
 
