package org.garret.serme.impl;

import java.util.Vector;
import org.garret.serme.*;

public class TtreePage implements ISerializable { 
    static final int maxItems = 64;
    static final int minItems = maxItems - 2; // minimal number of items in internal node

    TtreePage   left;
    TtreePage   right;
    int         balance;
    int         nItems;
    Object      item[];

    public TtreePage() {}

    public TtreePage(Object mbr) { 
        nItems = 1;
        item = new Object[maxItems];
        item[0] = mbr;
    }

    public boolean find(IComparator comparator, Object minValue, Object maxValue, Vector selection)
    { 
        int l, r, m, n;
        n = nItems;
        if (minValue != null) { 
            if (comparator.compareMemberWithKey(item[0], minValue) < 0) {       
                if (comparator.compareMemberWithKey(item[n-1], maxValue) < 0) { 
                    if (right != null) { 
                        return right.find(comparator, minValue, maxValue, selection); 
                    } 
                    return true;
                }
                for (l = 0, r = n; l < r;) { 
                    m = (l + r) >> 1;
                    if (comparator.compareMemberWithKey(item[m], minValue) < 0) {
                        l = m+1;
                    } else { 
                        r = m;
                    }
                }
                while (r < n) { 
                    if (maxValue != null
                        && comparator.compareMemberWithKey(item[r], maxValue) > 0)
                    { 
                        return false;
                    }
                    selection.addElement(item[r]);
                    r += 1;
                }
                if (right != null) { 
                    return right.find(comparator, minValue, maxValue, selection); 
                } 
                return true;    
            }
        }       
        if (left != null) { 
            if (!left.find(comparator, minValue, maxValue, selection)) { 
                return false;
            }
        }
        for (l = 0; l < n; l++) { 
            if (maxValue != null && comparator.compareMemberWithKey(item[l], maxValue) > 0) {
                return false;
            }
            selection.addElement(item[l]);
        }
        if (right != null) { 
            return right.find(comparator, minValue, maxValue, selection);
        }         
        return true;
    }
    
    public boolean contains(IComparator comparator, Object mbr)
    { 
        int l, r, m, n;
        n = nItems;
        if (comparator.compareMembers(item[0], mbr) < 0) {      
            if (comparator.compareMembers(item[n-1], mbr) < 0) { 
                if (right != null) { 
                    return right.contains(comparator, mbr); 
                } 
                return false;
            }
            for (l = 0, r = n; l < r;) { 
                m = (l + r) >> 1;
                if (comparator.compareMembers(item[m], mbr) < 0) {
                    l = m+1;
                } else { 
                    r = m;
                }
            }
            while (r < n) { 
                if (mbr == item[r]) { 
                    return true;
                }
                if (comparator.compareMembers(item[r], mbr) > 0) { 
                    return false;
                }
                r += 1;
            }
            if (right != null) { 
                return right.contains(comparator, mbr); 
            } 
            return false;       
        }
        if (left != null) { 
            if (left.contains(comparator, mbr)) { 
                return true;
            }
        }
        for (l = 0; l < n; l++) { 
            if (mbr == item[l]) { 
                return true;
            }
            if (comparator.compareMembers(item[l], mbr) > 0) {
                return false;
            }
        }
        if (right != null) { 
            return right.contains(comparator, mbr);
        }         
        return false;
    }

    
    public static final int OK         = 0;
    public static final int NOT_UNIQUE = 1;
    public static final int NOT_FOUND  = 2;
    public static final int OVERFLOW   = 3;
    public static final int UNDERFLOW  = 4;

    private final void store(IStorage storage) { 
        if (storage != null) { 
            storage.store(this);
        }
    }

    private final void remove(IStorage storage) { 
        if (storage != null) { 
            storage.remove(this);
        }
    }

    public int insert(IStorage storage, IComparator comparator, Object mbr, boolean unique, TtreePageReference ref) 
    { 
        int n = nItems;
        TtreePage pg;
        int diff = comparator.compareMembers(mbr, item[0]);
        if (diff <= 0) { 
            if (unique && diff == 0) { 
                return NOT_UNIQUE;
            }
            if ((left == null || diff == 0) && n != maxItems) { 
                //for (int i = n; i > 0; i--) item[i] = item[i-1];
                System.arraycopy(item, 0, item, 1, n);
                item[0] = mbr;
                nItems += 1;
                store(storage);
                return OK;
            } 
            if (left == null) { 
                left = new TtreePage(mbr);
            } else {
                pg = ref.pg;
                ref.pg = left;
                int result = left.insert(storage, comparator, mbr, unique, ref);
                if (result == NOT_UNIQUE) { 
                    return NOT_UNIQUE;
                }
                left = ref.pg;
                ref.pg = pg;
                if (result == OK) { 
                    store(storage);
                    return OK;
                }
            }
            int result = OK;
            if (balance > 0) { 
                balance = 0;
            } else if (balance == 0) { 
                balance = -1;
                result = OVERFLOW;
            } else { 
                TtreePage left = this.left;
                if (left.balance < 0) { // single LL turn
                    this.left = left.right;
                    left.right = this;
                    balance = 0;
                    left.balance = 0;
                    ref.pg = left;
                } else { // double LR turn
                    TtreePage right = left.right;
                    left.right = right.left;
                    right.left = left;
                    this.left = right.right;
                    right.right = this;
                    balance = (right.balance < 0) ? 1 : 0;
                    left.balance = (right.balance > 0) ? -1 : 0;
                    right.balance = 0;
                    ref.pg = right;
                    right.store(storage);
                }
                left.store(storage);
            }
            store(storage);
            return result;
        } 
        diff = comparator.compareMembers(mbr, item[n-1]);
        if (diff >= 0) { 
            if (unique && diff == 0) { 
                return NOT_UNIQUE;
            }
            if ((right == null || diff == 0) && n != maxItems) { 
                item[n] = mbr;
                nItems += 1;
                store(storage);
                return OK;
            }
            if (right == null) { 
                right = new TtreePage(mbr);
            } else { 
                pg = ref.pg;
                ref.pg = right;
                int result = right.insert(storage, comparator, mbr, unique, ref);
                if (result == NOT_UNIQUE) { 
                    return NOT_UNIQUE;
                }
                right = ref.pg;
                ref.pg = pg;
                if (result == OK) { 
                    store(storage);
                    return OK;
                }
            }
            int result = OK;
            if (balance < 0) { 
                balance = 0;
            } else if (balance == 0) { 
                balance = 1; 
                result = OVERFLOW;
            } else { 
                TtreePage right = this.right;
                if (right.balance > 0) { // single RR turn
                    this.right = right.left;
                    right.left = this;
                    balance = 0;
                    right.balance = 0;
                    ref.pg = right;
                } else { // double RL turn
                    TtreePage left = right.left;
                    right.left = left.right;
                    left.right = right;
                    this.right = left.left;
                    left.left = this;
                    balance = (left.balance > 0) ? -1 : 0;
                    right.balance = (left.balance < 0) ? 1 : 0;
                    left.balance = 0;
                    ref.pg = left;
                    left.store(storage);
                }
                right.store(storage);
            }
            store(storage);
            return result;
        }
        int l = 1, r = n-1;
        while (l < r)  {
            int i = (l+r) >> 1;
            diff = comparator.compareMembers(mbr, item[i]);
            if (diff > 0) { 
                l = i + 1;
            } else { 
                r = i;
                if (diff == 0) { 
                    if (unique) { 
                        return NOT_UNIQUE;
                    }
                    break;
                }
            }
        }
        // Insert before item[r]
        if (n != maxItems) {
            System.arraycopy(item, r, item, r+1, n-r);
            //for (int i = n; i > r; i--) item[i] = item[i-1]; 
            item[r] = mbr;
            nItems += 1;
            store(storage);
            return OK;
        } else { 
            Object reinsertItem;
            if (balance >= 0) { 
                reinsertItem = item[0];
                System.arraycopy(item, 1, item, 0, r-1);
                //for (int i = 1; i < r; i++) item[i-1] = item[i]; 
                item[r-1] = mbr;
            } else { 
                reinsertItem = item[n-1];
                System.arraycopy(item, r, item, r+1, n-r-1);
                //for (int i = n-1; i > r; i--) item[i] = item[i-1]; 
                item[r] = mbr;
            }
            store(storage);
            return insert(storage, comparator, reinsertItem, unique, ref);
        }
    }
       
    private int balanceLeftBranch(IStorage storage, TtreePageReference ref) 
    {
        int result = OK;
        if (balance < 0) { 
            balance = 0;
            result = UNDERFLOW;
        } else if (balance == 0) { 
            balance = 1;
        } else { 
            TtreePage right = this.right;
            if (right.balance >= 0) { // single RR turn
                this.right = right.left;
                right.left = this;
                if (right.balance == 0) { 
                    this.balance = 1;
                    right.balance = -1;
                    ref.pg = right;
                } else { 
                    balance = 0;
                    right.balance = 0;
                    ref.pg = right;
                    result = UNDERFLOW;
                }
            } else { // double RL turn
                TtreePage left = right.left;
                right.left = left.right;
                left.right = right;
                this.right = left.left;
                left.left = this;
                balance = left.balance > 0 ? -1 : 0;
                right.balance = left.balance < 0 ? 1 : 0;
                left.balance = 0;
                ref.pg = left;
                result = UNDERFLOW;
                left.store(storage);
            }
            right.store(storage);
        }
        store(storage);
        return result;
    }

    private int balanceRightBranch(IStorage storage, TtreePageReference ref) 
    {
       int result = OK;
        if (balance > 0) { 
            balance = 0;
            result = UNDERFLOW;
        } else if (balance == 0) { 
            balance = -1;
        } else { 
            TtreePage left = this.left;
            if (left.balance <= 0) { // single LL turn
                this.left = left.right;
                left.right = this;
                if (left.balance == 0) { 
                    balance = -1;
                    left.balance = 1;
                    ref.pg = left;
                } else { 
                    balance = 0;
                    left.balance = 0;
                    ref.pg = left;
                    result = UNDERFLOW;
                }
            } else { // double LR turn
                TtreePage right = left.right;
                left.right = right.left;
                right.left = left;
                this.left = right.right;
                right.right = this;
                balance = right.balance < 0 ? 1 : 0;
                left.balance = right.balance > 0 ? -1 : 0;
                right.balance = 0;
                ref.pg = right;
                result = UNDERFLOW;
                right.store(storage);
            }
            left.store(storage);
        }
        store(storage);
        return result;
    }
    
    public int remove(IStorage storage, IComparator comparator, Object mbr, TtreePageReference ref)
    {
        TtreePage pg;
        int n = nItems;
        int diff = comparator.compareMembers(mbr, item[0]);
        if (diff <= 0) { 
            if (left != null) { 
                pg = ref.pg;
                ref.pg = left;
                int h = left.remove(storage, comparator, mbr, ref);
                left = ref.pg;
                ref.pg = pg;
                if (h == UNDERFLOW) { 
                    return balanceLeftBranch(storage, ref);
                } else if (h == OK) { 
                    store(storage);
                    return OK;
                }
            }
        }
        diff = comparator.compareMembers(mbr, item[n-1]);
        if (diff <= 0) {            
            for (int i = 0; i < n; i++) { 
                if (item[i] == mbr) { 
                    if (n == 1) { 
                        if (right == null) { 
                            remove(storage);
                            ref.pg = left;
                            return UNDERFLOW;
                        } else if (left == null) { 
                            remove(storage);
                            ref.pg = right;
                            return UNDERFLOW;
                        } 
                    }
                    if (n <= minItems) { 
                        if (left != null && balance <= 0) {  
                            TtreePage prev = left;
                            while (prev.right != null) {                                 
                                prev = prev.right;
                            }
                            System.arraycopy(item, 0, item, 1, i);
                            //while (--i >= 0) { 
                            //    item[i+1] = item[i];
                            //}
                            item[0] = prev.item[prev.nItems-1];
                            pg = ref.pg;
                            ref.pg = left;
                            int h = left.remove(storage, comparator, item[0], ref);
                            left = ref.pg;
                            ref.pg = pg;
                            if (h == UNDERFLOW) {
                                h = balanceLeftBranch(storage, ref);
                            } else { 
                                store(storage);
                            }
                            return h;
                        } else if (right != null) { 
                            TtreePage next = right;
                            while (next.left != null) { 
                                next = next.left;
                            }
                            System.arraycopy(item, i+1, item, i, n-i-1);
                            //while (++i < n) { 
                            //    item[i-1] = item[i];
                            //}
                            item[n-1] = next.item[0];
                            pg = ref.pg;
                            ref.pg = right;
                            int h = right.remove(storage, comparator, item[n-1], ref);
                            right = ref.pg;
                            ref.pg = pg;
                            if (h == UNDERFLOW) {
                                h = balanceRightBranch(storage, ref);
                            } else { 
                                store(storage);
                            }
                            return h;
                        }
                    }
                    System.arraycopy(item, i+1, item, i, n-i-1);
                    //while (++i < n) { 
                    //    item[i-1] = item[i];
                    //}
                    item[n-1] = null;
                    nItems -= 1;
                    store(storage);
                    return OK;
                }
            }
        }
        if (right != null) { 
            pg = ref.pg;
            ref.pg = right;
            int h = right.remove(storage, comparator, mbr, ref);
            right = ref.pg;
            ref.pg = pg;
            if (h == UNDERFLOW) { 
                return balanceRightBranch(storage, ref);
            } else { 
                store(storage);
                return h;
            }
        }
        return NOT_FOUND;
    }


    public int toArray(Object[] arr, int index) { 
        if (left != null) { 
            index = left.toArray(arr, index);
        }
        for (int i = 0, n = index + nItems < arr.length ? nItems : arr.length - index; i < n; i++) { 
            arr[index++] = item[i];
        }
        if (right != null) { 
            index = right.toArray(arr, index);
        }
        return index;
    }

    public void prune(IStorage storage) { 
        if (left != null) { 
            left.prune(storage);
        }
        if (right != null) { 
            right.prune(storage);
        }
        storage.remove(this);
    }

    public void writeObject(IOutputStream out) { 
        out.writeObject(left);
        out.writeObject(right);
        out.writeByte((byte)balance);
        out.writeShort((short)nItems);
        out.writeArrayOfObject(item);
    }

    public void readObject(IInputStream in) { 
        left = (TtreePage)in.readObject();
        right = (TtreePage)in.readObject();
        balance = in.readByte();
        nItems = in.readShort();
        item = in.readArrayOfObject();
    }
}
