package org.garret.serme.rms;

import org.garret.serme.*;
import org.garret.serme.impl.*;
import javax.microedition.rms.*;

class RmsObjectInputStream extends ObjectDataInputStream {
    RmsObjectInputStream() {
        super(null);
    }

    Object getRootObject(RmsStorage storage) { 
        RecordStore store = storage.store;
        String className = null;
        try { 
            byte[] record = store.getRecord(RmsStorage.METADATA_RID);
            rootOid = Converter.unpackInt(record, 4);
            int nClasses = Converter.unpackShort(record, 8);
            classNames = Converter.unpackString(record, 10);        
            classes = new Class[nClasses];
            oidToObj = new OidToObj();
            int beg = 0;
            for (int i = 0; i < nClasses; i++) { 
                int end = classNames.indexOf(';', beg);
                className = classNames.substring(beg, end);
                classes[i] = Class.forName(className);
                beg = end + 1;
            }
            RecordEnumeration e = store.enumerateRecords(null, null, false);
            /* Retriving records takes too much time
             * and size of RMS storage is very limited. 
             * So just extract all data in temporary array
             */
            byte[][] buffer = new byte[store.getNumRecords()-1][];
            int bp = 0;
            while (e.hasNextElement()) { 
                record = e.nextRecord();
                int oid = Converter.unpackInt(record, 0);
                if (oid != 0) {
                    buffer[bp++] = record;
                    int cid = Converter.unpackShort(record, 4);
                    oidToObj.put(oid, classes[cid-1].newInstance());
                }
            }
            for (int i = 0; i < bp; i++) { 
                arr = buffer[i];
                pos = 6;
                int oid = Converter.unpackInt(arr, 0);
                Object obj = oidToObj.get(oid);
                unpackObject(obj);
                if (obj instanceof Persistent) {
                    ((Persistent)obj).setStorage(storage);
                }
            }
                 
            /*
            while (e.hasNextElement()) { 
                record = e.nextRecord();
                int oid = Converter.unpackInt(record, 0);
                if (oid != 0) {
                    int cid = Converter.unpackShort(record, 4);
                    oidToObj.put(oid, classes[cid-1].newInstance());
                }
            }
            e.reset();
            while (e.hasNextElement()) { 
                record = e.nextRecord();
                int oid = Converter.unpackInt(record, 0);
                if (oid != 0) { 
                    arr = record;
                    pos = 6;
                    Object obj = oidToObj.get(oid);
                    unpackObject(obj);
                    if (obj instanceof Persistent) {
                        ((Persistent)obj).setStorage(storage);
                    }
                }
            }
            */
        } catch (RecordStoreException x) {
            throw new SermeException("Storage access error: " + x);
        } catch (ClassNotFoundException x) {
            throw new SermeException("Class " + className + " not found");
        } catch (IllegalAccessException x) { 
            throw new SermeException("Class of instantiated object is not accessible: " + x);
        } catch (InstantiationException x) { 
            throw new SermeException("Failed to instantiate object: " + x);
        }
        return oidToObj.get(rootOid);
    }

    protected Object deserializeObject() { 
        throw new SermeException("Invalid serialization data");
    }

    protected Object getObject(int oid) { 
        return oidToObj.get(oid);
    }

    OidToObj oidToObj;
    Class[]  classes;
    String   classNames;    
    int      rootOid;
}
