package org.garret.serme.rms;

import java.util.Vector;
import javax.microedition.rms.*;

import org.garret.serme.*;
import org.garret.serme.impl.*;


class RmsObjectOutputStream extends ObjectDataOutputStream {

    public RmsObjectOutputStream(RmsStorage storage, int rootOid) { 
        this.storage = storage;
        store = storage.store;
        objToOid = storage.objToOid;
        classToOid = storage.classToOid;
        classNames = storage.classNames;
        this.rootOid = rootOid;
        try { 
            lastOid = store.getNextRecordID();
        } catch (RecordStoreException x) { 
            throw new SermeException("Failed to get next record ID");
        }
    }
        
    public void close() {}

    protected void serializeObject(Object obj) { 
        int oid;
        if (obj == null) { 
            oid = 0;
        } else { 
            oid = objToOid.get(obj);
            if (oid == 0) {
                oid = lastOid++;
                objToOid.put(obj, oid);
                objects.addElement(obj);
            }
        }
        extend(5);        
        arr[used++] = Types.OBJECT;
        Converter.packInt(arr, used, oid);
        used += 4;
    }

    void storeRoot(Object obj) { 
        rootOid = objToOid.get(obj);
        if (rootOid == 0) { 
            rootOid = lastOid;
        }
        storeObject(obj);
        metadataChanged = true;
    }
            
    void storeObject(Object obj) { 
        objects = new Vector();
        storeObject0(obj, false);
        for (int i = 0; i < objects.size(); i++) { 
            storeObject0(objects.elementAt(i), true);
        }
        if (metadataChanged) {
            updateMetadata();
        }
        objects = null;
    }

    void storeObject0(Object obj, boolean create) { 
        int oid = objToOid.get(obj);
        if (oid == 0) { 
            oid = lastOid++;
            objToOid.put(obj, oid);
            create = true;
        }
        Class c = obj.getClass();
        int cid = classToOid.get(c);
        if (cid == 0) { 
            cid = classToOid.size()+1;            
            classToOid.put(c, cid);
            classNames.append(c.getName());
            classNames.append(';');
            metadataChanged = true;
        }
        extend(6);
        Converter.packInt(arr, 0, oid);
        Converter.packShort(arr, 4, (short)cid);
        used = 6;
        packObject(obj);
        if (obj instanceof Persistent) { 
            ((Persistent)obj).setStorage(storage);
        }
        try { 
            if (create) { 
                int newId = store.addRecord(arr, 0, used);
                if (newId != oid) { 
                    throw new SermeException("Internal error");
                }
            } else { 
                store.setRecord(oid, arr, 0, used);
            }
        } catch (RecordStoreException x) { 
            throw new SermeException("Failed to store object: " + x);
        }
    }
    
    void updateMetadata() {         
        byte[] record = new byte[12 + classNames.length()*2];
        Converter.packInt(record, 4, rootOid);
        Converter.packShort(record, 8, (short)classToOid.size());
        Converter.packString(record, 10, classNames.toString());
        try { 
            store.setRecord(RmsStorage.METADATA_RID, record, 0, record.length);
        } catch (RecordStoreException x) { 
            throw new SermeException("Failed to update metadata: " + x);
        }
        metadataChanged = false;
    }


    Vector       objects;
    ObjToOid     objToOid;
    ObjToOid     classToOid;
    StringBuffer classNames;
    boolean      metadataChanged;
    RecordStore  store;
    RmsStorage   storage;
    int          rootOid;
    int          lastOid;
}