package org.garret.serme.rms;

import java.util.*;
import javax.microedition.rms.*;

import org.garret.serme.*;
import org.garret.serme.impl.*;

/**
 * Storage implementation on top of javax.microedition.rms package.
 * This implementation should be used for convenient fetching/storing persistent data in
 * J2ME application. 
 */
public class RmsStorage implements IStorage { 
    static final int METADATA_RID = 1;

    /**
     * Storage constructor
     * @param name name of the storage (passed to <Code>javax.microedition.rms.RecordStore.openStore</code>)
     */
    public RmsStorage(String name) { 
        this.name = name;
    }

    protected void checkIfOpened(boolean shouldBeOpened) { 
        if (opened != shouldBeOpened) { 
            throw new SermeException("Incorrect storage state");
        }
    }

    public void store(Object obj) { 
        checkIfOpened(true);
        output.storeObject(obj);
    }

    public void remove(Object obj) { 
        checkIfOpened(true);
        int oid = objToOid.remove(obj);
        if (oid == 0) { 
            throw new SermeException("Object not persistent");
        }
        try { 
            store.deleteRecord(oid);
        } catch (RecordStoreException x) { 
            throw new SermeException("Failed to remove object: " + x);
        }            
    }

    public boolean open() { 
        return open0(false);
    }

    public boolean create() { 
        return open0(true);
    }

    public boolean remove() { 
        checkIfOpened(false);
        try { 
            RecordStore.deleteRecordStore(name);
        } catch (RecordStoreNotFoundException x) { 
            return false;
        } catch (RecordStoreException x) { 
            throw new SermeException("Failed to delete storage: " + x);
        }
        return true;
    }

    protected boolean open0(boolean create) {         
        checkIfOpened(false);
        store = null;
        try { 
            store = RecordStore.openRecordStore(name, create);
            int rootOid = 0;
            if (store.getNumRecords() == 0) { 
                byte[] emptyStore = new byte[12];
                store.addRecord(emptyStore, 0, emptyStore.length);                
                classNames = new StringBuffer();
                objToOid = new ObjToOid();
                classToOid = new ObjToOid();
            } else { 
                RmsObjectInputStream input = new RmsObjectInputStream();
                root = input.getRootObject(this);
                classNames = new StringBuffer(input.classNames);
                rootOid = input.rootOid;
                objToOid = new ObjToOid(input.oidToObj);
                Class[] classes = input.classes;
                classToOid = new ObjToOid(classes.length);
                for (int i = 0; i < classes.length; i++) { 
                    classToOid.put(classes[i], i+1);
                }
            }
            output = new RmsObjectOutputStream(this, rootOid);
            opened = true;
        } catch (RecordStoreNotFoundException x) { 
            return false;
        } catch (RecordStoreException x) { 
            if (store != null) { 
                try { 
                    store.closeRecordStore();
                } catch (RecordStoreException e) {}
            }
            throw new SermeException("Failed to open storage: " + x);
        }
        return true;
    }

    public void close() { 
        checkIfOpened(true);
        try { 
            store.closeRecordStore();
        } catch (RecordStoreException x) { 
            throw new SermeException("Failed to close storage: " + x);
        }
        opened = false;
    }
        
    public Object getRoot() { 
        checkIfOpened(true);
        return root;
    }

    public void setRoot(Object root) { 
        checkIfOpened(true);
        output.storeRoot(root);
        this.root = root;
    }

    public Enumeration enumerateObjects() { 
        return objToOid.keys();
    }

    ObjToOid     objToOid;
    ObjToOid     classToOid;
    StringBuffer classNames;
    RecordStore  store;
    Object       root;
    boolean      opened;
    String       name;
    RmsObjectOutputStream output;
}